﻿/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Tester, MultiplicationsGenerator, AdditionsGenerator */

var tester = null;

function clearAnswer() {
    var element = document.getElementById("answer");
    element.value = "";
    element = document.getElementById("correctionDiv");
    element.className = "hidden";
}

function clearCounters() {
    var counterSpan;
    counterSpan = document.getElementById("correct");
    counterSpan.firstChild.data = "0";
    counterSpan = document.getElementById("total");
    counterSpan.firstChild.data = "0";
}

function askNextQuestion() {
    clearAnswer();
    var element = document.getElementById("question");
    element.firstChild.data = tester.nextQuestion();
    
    element = document.getElementById("answer");
    element.disabled = false;
    element.focus();
}

function loadHandler() {
    clearAnswer();
    clearCounters();
}

function startClickHandler() {
    var title = "";
    var questionsGenerator = null;

    document.getElementById("test").className = "exam";

    var tableTypeElement = document.getElementById("multiplicationOption");
    if (tableTypeElement.checked) {
        questionsGenerator = new MultiplicationsGenerator();
        title = "Умножение: Таблица с ";
    }
    else {
        questionsGenerator = new AdditionsGenerator();
        title = "Събиране с ";
    }

    var table = document.getElementById("table").value;
    questionsGenerator.parameters.table.setValue(table);

    title += table;
    document.getElementById("testTitle").firstChild.data = title;

    tester = new Tester(questionsGenerator);
    
    clearCounters();

    var answerInput = document.getElementById("answer");
    answerInput.disabled = false;
    answerInput.focus();

    askNextQuestion();
}

function incrementCounter(id) {
    var counterSpan = document.getElementById(id);
    counterSpan.firstChild.data = (counterSpan.firstChild.data - 0) + 1;
}

function fillCell(cell, value) {
    cell.firstChild.data = value;
}

function showReview() {
    var rows;
    var row;
    var question;
    var duration;
    var maxDuration;
    var faults = 0;
    var scoreDiv;

    document.getElementById("test").className = "result";
    document.getElementById("correctionDiv").className = "hidden";
    
    maxDuration = document.getElementById("max").value;
    rows = document.getElementsByTagName("tr");
    for (var i = 0; i < tester.questions.length; i += 1) {
        row = rows[i + 1];
        question = tester.questions[i];
        fillCell(row.childNodes[0], question.getStatement());
        fillCell(row.childNodes[1], question.answeredValue);
        fillCell(row.childNodes[2], question.getResult());

        duration = question.getDuration() / 1000;
        duration = (Math.round(duration * 10) / 10);
        fillCell(row.childNodes[3],
                 duration + " сек." +
                 (duration > 1 ? " " : ""));

        row.className = "";
        if (duration > maxDuration) {
            row.className = "late";
        }

        if (!question.isCorrect()) {
            row.className = "wrong";
            faults += 1;
        }
    }

    scoreDiv = document.getElementById("score");
    switch (faults) {
    case 0:
        scoreDiv.className = "zeroFault";
        break;
    case 1:
        scoreDiv.className = "oneFault";
        break;
    default:
        scoreDiv.className = "manyFaults";
        break;
    }

    document.getElementById("test").className = "result";
}

function doAfterTimeout(funct, milliseconds) {
    if (milliseconds === 0) {
        funct();
    }
    else {
        setTimeout(funct, milliseconds);
    }
}

function keyPressHandler(event) {
    var answer = document.getElementById("answer");
    var yesNo = document.getElementById("yesNo");
    var correctAnswer = document.getElementById("correctAnswer");

    if (event.keyCode === 13) { // Enter key pressed.
        answer.disabled = true;

        if (tester.answerCurrent(answer.value)) {
            yesNo.firstChild.data = "ДА!";
            yesNo.className = "zeroFault";
            correctAnswer.className = "hidden";
            correctAnswer.firstChild.data = "";
            document.getElementById("correctionDiv").className = "";
            incrementCounter("correct");
        }
        else {
            yesNo.firstChild.data = "НЕ!";
            yesNo.className = "manyFaults";
            correctAnswer.className = "";
            correctAnswer.firstChild.data =
                tester.getCurrentQuestionCorrection();
            document.getElementById("correctionDiv").className = "";
        }
        incrementCounter("total");
        
        var delay = document.getElementById("delay").value;
        delay = delay - 0;
        if (tester.lastReached()) {
            doAfterTimeout(showReview, delay * 1000);            
        }
        else {
            doAfterTimeout(askNextQuestion, delay * 1000);
        }

        return false;
    }
}
