/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern
  assert, assertEquals, assertNotEquals, assertNotUndefined, assertFalse,
  debug */

/**
 * WARNING: tests in this page must be executed in the specified order.
 */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testGetTestDocument",
        "testFormClassAfterLoad",
        "testFormClassAfterStart",
        "testFormClassAfterChickenTest"
    ];
}

function getFrameDocument(frameId) {
    // W3C-compliant method.
    var frameDocument = document.getElementById(frameId).contentDocument;

    if (!frameDocument) {
        // IE-specific method.
        frameDocument = document.frames[frameId].document;
    }
    return frameDocument;
}

function getTestDocument() {
    return getFrameDocument("appFrame");
}

function simulateEnterKeyPress() {
    var d = getTestDocument();
    var answerInput = d.getElementById("answer");
    var e = null;
    if (window.KeyEvent) {
        // Mozilla-specific.
        e = d.createEvent("KeyEvents");
        e.initKeyEvent("keypress", true, true, null, false, false, false, false, 13, 0);
        answerInput.dispatchEvent(e);
    }
    else {
        // IE-specific.
        e = d.createEventObject();
        e.keyCode = 13;
        answerInput.fireEvent("onkeypress", e);
    }
}

function testGetTestDocument() {
    var d = getTestDocument();
    assertNotUndefined("Invalid object returned by getTestDocument.",
		       d.getElementById);
}

/**
 * Test 1.
 */
function testFormClassAfterLoad() {
    var d = getTestDocument();
    
    assertEquals("Incorrect class of test frame after document load.",
		 "config", d.getElementById("test").className);
}

/**
 * Test 2.
 */
function testFormClassAfterStart() {
    var d = getTestDocument();
    d.getElementById("table").value = "3";
    d.getElementById("delay").value = "0";
    d.getElementById("start").onclick();

    assertEquals("Incorrect class of test frame after document load.",
		 "exam", d.getElementById("test").className);
}

/**
 * Test 3.
 */
function testFormClassAfterChickenTest() {
    var d = getTestDocument();
    var testDiv = d.getElementById("test");

    for (var i = 0; i < 19; i += 1) {
        simulateEnterKeyPress();
        debug("KeyPress event simulated");
    }

    assertEquals("Incorrect class of test frame after complete chicken test.",
		 "result", testDiv.className);
}
