/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Multiplication, MultiplicationsGenerator, QuestionsGenerator,
  JsUnitException, assertEquals, assertFalse, assertNotEquals, assertNotUndefined, debug, fail */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testCorrectInheritanceFromQuestionsGenerator",
        "testAvailableParameters",
        "testParameterTableDescription",
        "testParameterTableSetValueLetter",
        "testParameterTableSetValueStringConvertibleToNatural",
        "testMaxDifferentQuestions",
        "testGenerateQuestionsParameterNotSet",
        "testGenerateQuestionsReturnsCorrectSize",
        "testGenerateQuestionsReturnsMultiplications",
        "testGenerateQuestionsTooFewRequested",
        "testGenerateQuestionsTooManyRequested",
        "testGenerateQuestionsMaxAllDifferent"
    ];
}

function testCorrectInheritanceFromQuestionsGenerator() {
    var multGen = new MultiplicationsGenerator();
    var questGen = new QuestionsGenerator();

    assertNotUndefined("generateQuestions method", multGen.generateQuestions);
    assertEquals("Should not override generateQuestions method.",
		 questGen.generateQuestions, multGen.generateQuestions);
    assertNotUndefined("questionExists method", multGen.questionExists);
    assertEquals("Should not override questionExists method.",
		 questGen.questionExists, multGen.questionExists);

    assertNotUndefined("maxDifferentQuestions property", multGen.maxDifferentQuestions);
    assertNotEquals("Should override maxDifferentQuestions property.",
		    questGen.maxDifferentQuestions, multGen.maxDifferentQuestions);
    assertNotUndefined("generateNewQuestion method", multGen.generateNewQuestion);
    assertNotEquals("Should override generateNewQuestion method.",
                    questGen.generateNewQuestion, multGen.generateNewQuestion);
}

function testAvailableParameters() {
    var multGen = new MultiplicationsGenerator();
    assertNotUndefined("Parameters list should be defined.",
                       multGen.parameters);
    assertNotUndefined("Parameter with ID 'table' should exist",
                       multGen.parameters["table"]);
}

function testParameterTableDescription() {
    var multGen = new MultiplicationsGenerator();
    var paramTable = multGen.parameters["table"];

    assertEquals("Incorrect description of table parameter.",
                 "Table de :", paramTable.description);
}

function testParameterTableSetValueLetter() {
    var multGen = new MultiplicationsGenerator();
    var paramTable = multGen.parameters["table"];
    try {
        paramTable.setValue("a");
        fail("Should fail to set value.");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testParameterTableSetValueStringConvertibleToNatural() {
    var multGen = new MultiplicationsGenerator();
    var paramTable = multGen.parameters["table"];

    paramTable.setValue("3");
    assert("Value should have been converted into a number.",
           paramTable.value === 3);
}

function testMaxDifferentQuestions() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    assertEquals(19, multGen.maxDifferentQuestions);
}

function testGenerateQuestionsParameterNotSet() {
    var multGen = new MultiplicationsGenerator();

    try {
        multGen.generateQuestions(1);
        fail("Should fail to generate questions.");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testGenerateQuestionsReturnsCorrectSize() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    var questions = multGen.generateQuestions(2);
    assertEquals("Number of generated questions",
                 2, questions.length);
}

function testGenerateQuestionsReturnsMultiplications() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    var questions = multGen.generateQuestions(1);
    assertEquals("Type of questions generated",
                 Multiplication, questions[0].constructor);
}

function testGenerateQuestionsTooFewRequested() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    try {
        multGen.generateQuestions(20);
        fail("Generating 20 questions should be impossible");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
        debug(e);
    }
}

function testGenerateQuestionsTooManyRequested() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    try {
        multGen.generateQuestions(-1);
        fail("Generating -1 question should be impossible");
    }
    catch (e) {
        if (e.constructor === JsUnitException) {
            throw e;
        }
    }
}

function testGenerateQuestionsMaxAllDifferent() {
    var multGen = new MultiplicationsGenerator();
    multGen.parameters["table"].setValue("7");

    var questions = multGen.generateQuestions(multGen.maxDifferentQuestions);
    for (var i = 0; i < multGen.maxDifferentQuestions; i += 1) {
        for (var j = 0; j < multGen.maxDifferentQuestions; j += 1) {
            if (i !== j) {
                assertFalse("Duplicate question", questions[i].equals(questions[j]));
            }
        }
    }
}

